from taktile_auth.entities.permission import Permission, PermissionDefinition
from taktile_auth.entities.resource import Resource, ResourceDefinition
from taktile_auth.entities.role import Role, RoleDefinition



"""
Taktile Auth
"""
import pkg_resources

__version__ = pkg_resources.get_distribution(
    __name__.split(".", maxsplit=1)[0]
).version

from taktile_auth.entities import Permission, Role  # noqa: 401
from taktile_auth.parser import RESOURCES, ROLES  # noqa: 401
from taktile_auth.utils import build_query, parse_role  # noqa: 401



import pathlib

import pkg_resources

ROLE_PATH = pathlib.Path(
    pkg_resources.resource_filename("taktile_auth", "../assets/roles.yaml")
)
RESOURCE_PATH = pathlib.Path(
    pkg_resources.resource_filename("taktile_auth", "../assets/resources.yaml")
)



import re
import typing as t

import pydantic

from taktile_auth.enums import Wildcard

PARTIAL_RE = re.compile(r"(^[^\*]*$)|(^[^\*]*\*$)")


def is_full_wildcard(v):
    assert v == "*" or "*" not in v
    return v


def is_partial_wildcard(v):
    assert re.fullmatch(PARTIAL_RE, v)
    return v


def is_full_specified(v):
    assert "*" not in v
    return v


WILDCARD_CHECK = {
    Wildcard.allowed: is_full_wildcard,
    Wildcard.partial: is_partial_wildcard,
    Wildcard.not_allowed: is_full_specified,
}


class Resource(pydantic.BaseModel):
    def __hash__(self):
        return hash(tuple(self.__dict__.values()))


class ResourceDefinition(pydantic.BaseModel):
    resource_name: str
    args: t.Dict[str, Wildcard]

    def get_resource(self) -> Resource:
        fields = {field_name: (str, ...) for field_name in self.args.keys()}
        validators = {
            f"{field_name}_validator": (
                pydantic.validator(field_name, allow_reuse=True)(
                    WILDCARD_CHECK[check]
                )
            )
            for field_name, check in self.args.items()
        }
        return pydantic.create_model(
            self.resource_name,
            **fields,
            __validators__=validators,
            __base__=Resource,
        )



import typing as t

from taktile_auth.entities import Permission, Role
from taktile_auth.parser import RESOURCES, ROLES
from taktile_auth.parser.utils import parse_permission


def build_query(query: str) -> Permission:

    parsed_query = parse_permission(query)
    fields = list(RESOURCES[parsed_query["resource_name"]].args.keys())
    resource_vals = {
        fields[x]: parsed_query["resource_args"][x] for x in range(len(fields))
    }
    return Permission(
        actions=set(parsed_query["actions"]),
        resource=RESOURCES[parsed_query["resource_name"]].get_resource()(
            **resource_vals
        ),
    )


def parse_role(role_signature):
    # deployment_admin/RohanMishra97,607da278-5d13-4479-ad47-ccf094013c3a,*
    role_name, sep, role_args_str = role_signature.partition("/")
    role_args = role_args_str.split(",")
    role_def = ROLES[role_name]
    return ROLES[role_name].build(**dict(zip(role_def.args, role_args)))


class InsufficientRightsException(Exception):
    pass


def _is_allowed(query: str, roles: t.List[Role]):
    query = build_query(query)
    if not any(query in role for role in roles):
        raise InsufficientRightsException


def assert_access(query: t.Union[str, t.Sequence[str]], roles: t.List[Role]):
    if isinstance(query, str):
        _is_allowed(query, roles)
    else:
        for q in query:
            _is_allowed(q, roles)



import re
import typing as t

import pydantic

from taktile_auth.entities.resource import Resource, ResourceDefinition
from taktile_auth.enums import Action


class Permission(pydantic.BaseModel):
    actions: t.Set[Action]
    resource: Resource

    def __contains__(self, query: "Permission") -> bool:
        action_match = query.actions.difference(self.actions) == set()
        resource_match = True
        for field in self.resource.dict().keys():
            allowed = getattr(self.resource, field)
            queried = getattr(query.resource, field)
            if not re.fullmatch(
                allowed.replace("*", ".*"), queried
            ):  # TODO - Beware of regex fails
                resource_match = False
                break
        return action_match and resource_match

    def __repr__(self) -> str:
        actions = "+".join(sorted(self.actions))
        resource_name = type(self.resource).__name__
        vals = ",".join(self.resource.dict().values())
        return f"{actions}:{resource_name}/{vals}"


Permission.update_forward_refs()


class PermissionDefinition(pydantic.BaseModel):
    actions: t.Set[Action]
    resource_definition: ResourceDefinition

    def build(self, **kwargs) -> Permission:
        extra_args = {
            arg: "*"
            for arg in set(self.resource_definition.args.keys()).difference(
                kwargs
            )
        }
        return Permission(
            actions=self.actions,
            resource=self.resource_definition.get_resource()(
                **kwargs, **extra_args
            ),
        )



import taktile_auth.settings as settings
from taktile_auth.parser.yaml_parsing import (
    parse_resource_yaml,
    parse_role_yaml,
)

RESOURCES = parse_resource_yaml(settings.RESOURCE_PATH)
ROLES = parse_role_yaml(settings.ROLE_PATH, settings.RESOURCE_PATH)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['taktile_auth', 'taktile_auth.entities', 'taktile_auth.parser']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0', 'pydantic>=1.8.2,<2.0.0']

setup_kwargs = {
    'name': 'taktile-auth',
    'version': '0.0.1a8',
    'description': 'Auth Package for Taktile',
    'long_description': None,
    'author': 'Taktile GmbH',
    'author_email': 'devops@taktile.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7.1,<4.0.0',
}


setup(**setup_kwargs)
